require './lib/named_vector.rb'
describe 'Initialization' do
	it 'should have all the dimensions given in the initilization' do
		dimensions = ["x","y"]
		vector = NamedVector.new(*dimensions)
		vector.dimensions.should include(*dimensions)
		dimensions.each do |dim|
			vector.should respond_to(dim.to_sym).with(0).arguments
		end
	end

	it 'should initialize from a hash as well' do
		hash_vector = {"x" => 1.0, "y" => 2.0, "z" => 0.0}
		vector = NamedVector.new(hash_vector)
		hash_vector.keys.should eql vector.keys.to_a
		hash_vector.keys.each do |key|
			vector.should respond_to(key.to_sym).with(0).arguments
			vector[key].should eql hash_vector[key]
		end
	end

	it 'should allow dot products between vectors' do
		dimensions = ["x","y"]
		vectorA = NamedVector.new(:x => 1, :y => 1)
		vectorB = NamedVector.new(:x => 1)
		(vectorA*vectorB).should eql 1.0
	end

	it 'should allow dot products between mismatched vectors' do
		vectorA = NamedVector.new(           "x" => 0, "z" => 2)
		vectorB = NamedVector.new("b" => 10,           "z" => 1)
		vectorA.x = 0
		vectorB.z = 1
		vectorA.z = 2
		vectorB.a = 10
		(vectorA*vectorB).should eql 2.0
	end

	it 'should allow dot products between vectors and hashes' do
		vectorA = NamedVector.new("x","y","z")
		vectorB = {:x => 1, :y => 2, :z => 3}
		vectorA.x = 1
		vectorA.y = 1
		vectorA.z = 1
		(vectorA*vectorB).should eql 6.0
	end

	it 'should normalize vectors' do
		vector = NamedVector.new("x","y","z")
		vector.x = 1
		vector.y = 1
		vector.z = 1
		vector.normalize
		vector.norm.should be_within(0.01).of 1.0
	end

	it 'should allow sums of vectors to be used for initilization' do
		vector = NamedVector.new(%w(a a a b b))
		vector.a.should eq 3
		vector.b.should eq 2
	end
end